function [m,meanP]=forestclusterdemo()

%% generate data
clear;
dataType = 2;

%0 -- gaussian data
%1 -- spiral
%2 -- two rings


switch dataType
    
    case 0
    % generate some random gaussian like data
        rand('state', 0);
        randn('state', 0);
        N= 100;
        clnum = 3;
        Nall = clnum*N;
    
        D= 2;

        X1 = mgd(N, D, [4 3], 0.5*[2 -1;-1 2]);
        X2 = mgd(N, D, [1 1], 0.5*[2 1;1 1]);
        X3 = mgd(N, D, [3 -3], 0.2*[1 0;0 4]);

        X= [X1; X2; X3];
        X= bsxfun(@rdivide, bsxfun(@minus, X, mean(X)), var(X));
        Y= [ones(N, 1); ones(N, 1)*2; ones(N, 1)*3];

        scatter(X(:,1), X(:,2), 20, Y)

    case 1
    % generate twirl data!

        N= 100;
        clnum = 3;
        Nall = clnum*N;
    
        t = linspace(0.5, 2*pi, N);
        x = t.*cos(t)  + rand(1,N)*0.1;
        y = t.*sin(t) + rand(1,N)*0.1;

        t = linspace(0.5, 2*pi, N);
        x2 = t.*cos(t+2) + rand(1,N)*0.1;
        y2 = t.*sin(t+2) + rand(1,N)*0.1;

        t = linspace(0.5, 2*pi, N);
        x3 = t.*cos(t+4) + rand(1,N)*0.1;
        y3 = t.*sin(t+4) + rand(1,N)*0.1;

        X= [[x' y']; [x2' y2']; [x3' y3']];
        X= bsxfun(@rdivide, bsxfun(@minus, X, mean(X)), var(X));
        Y= [ones(N, 1); ones(N, 1)*2; ones(N, 1)*3];

        scatter(X(:,1), X(:,2), 20, Y)
    case 2
    % generate ring data
        N=300;
        clnum = 2;
        Nall = clnum*N;
        t=rand(N,1)*2*pi;
    
        r1 = 8;
        r2 = 2;
    
        x1 = cos(t)*r1 + rand(N,1);
        y1 = sin(t)*r1 + rand(N,1);
    
        t=rand(N,1)*2*pi;
        x2 = cos(t)*r2 + rand(N,1);
        y2 = sin(t)*r2 + rand(N,1);
    
        X = [x1 y1; x2 y2];
        Y = [ones(N,1); 2*ones(N,1)];
    
        scatter(X(:,1), X(:,2), 20, Y)
        
end
%% now try to create clustering RF


ids1 = randi(size(X,1),[size(X,1),1]);
ids2 = randi(size(X,1),[size(X,1),1]);

newX = [X(ids1,1) X(ids2,2)];

tX = [X; newX];
tY = [ones(size(X,1),1); 2*ones(size(X,1),1)];

%% classify

rand('state', 0);
randn('state', 0);

opts= struct;
opts.depth= 5;
opts.numTrees= 400;
opts.numSplits= 5;
opts.verbose= true;
opts.saveIds = 1;
opts.classifierID= 2; % weak learners to use. Can be an array for mix of weak learners too
opts.minscount = 20;

tic;
[m,sample2node]= forestTrain(tX, tY, opts);
sample2node = sample2node(1:size(X,1),:);
meanP = s2n2P(sample2node);

meanP1 = meanP(1:Nall,1:Nall);
meanP = sqrt(1-meanP1);


meanPv = [];

%convert similarities to a vector
for i=1:(size(meanP,1)-1)
    meanPv = [meanPv meanP(i+1:end,i)'];
end

%[Medoids,clusvector,DIST]=KMedoidshort(meanP,clnum,[],[]);

%% hierarchical clustering (:D))
D = linkage(meanPv);

figure(2);

cla(gca);
dendrogram(D);
title('RF')
%c = cluster(D,'maxclust',clnum);
[c,optLevel,agg] = optHieClustering(D,clnum); %c can contain more then clnum classes


if (~agg)
    warning('could not find enough clusters of the appropriate size')
end

%% spectral clustering.. (does it make sense at all???)
% [C, L, U] = SpectralClustering(meanP1, clnum, 2);
% csc = zeros(size(meanP1,1),1);
% for i=1:clnum
%     csc(C(:,i)==1) = i;
% end
% 
figure(1);
subplot(1,3,1);
cla(gca)
plotClustering(tX,c)
title('RF+hierarchical clustering')
% 
% subplot(1,3,2);
% cla(gca)
% plotClustering(tX,csc)
% title('RF+spectral clustering')
% 
% %% spectral clustering on eucledian metrics
% 
% dist = SimGraph_NearestNeighbors(X',10,1);
% [C, L, U] = SpectralClustering(dist, clnum, 2);
% csc = zeros(size(dist,1),1);
% for i=1:clnum
%     csc(C(:,i)==1) = i;
% end
% 
% subplot(1,3,3);
% cla(gca)
% plotClustering(tX,csc)
% title('Knn + Spectral clustering')

end

function plotClustering(tX,cl)
    clnum = length(unique(cl));
    hold on
    for i=1:clnum
        plot(tX(cl==i,1),tX(cl==i,2),'o','MarkerFaceColor',rand(1,3),'MarkerEdgeColor','none','MarkerSize',10)
    end
end

